@@ -1,35 +1,37 @@
 import logging
 import os
-import pickle
 import warnings
 from itertools import cycle
 from typing import List, Optional, Union
 
 import numpy as np
 import torch
-from anndata import AnnData, read
+from anndata import AnnData
 from torch.utils.data import DataLoader
 
-from scvi import _CONSTANTS
-from scvi.data import transfer_anndata_setup
+from scvi import REGISTRY_KEYS
+from scvi.data import AnnDataManager
+from scvi.data._compat import manager_from_setup_dict
+from scvi.data._constants import _MODEL_NAME_KEY, _SETUP_ARGS_KEY
+from scvi.data.fields import CategoricalObsField, LayerField
 from scvi.dataloaders import DataSplitter
-from scvi.model._utils import _get_var_names_from_setup_anndata, parse_use_gpu_arg
+from scvi.model._utils import _init_library_size, parse_use_gpu_arg
 from scvi.model.base import BaseModelClass, VAEMixin
 from scvi.train import Trainer
+from scvi.utils import setup_anndata_dsp
 
 from ._module import JVAE
 from ._task import GIMVITrainingPlan
+from ._utils import _load_saved_gimvi_files
 
 logger = logging.getLogger(__name__)
 
 
 def _unpack_tensors(tensors):
-    x = tensors[_CONSTANTS.X_KEY].squeeze_(0)
-    local_l_mean = tensors[_CONSTANTS.LOCAL_L_MEAN_KEY].squeeze_(0)
-    local_l_var = tensors[_CONSTANTS.LOCAL_L_VAR_KEY].squeeze_(0)
-    batch_index = tensors[_CONSTANTS.BATCH_KEY].squeeze_(0)
-    y = tensors[_CONSTANTS.LABELS_KEY].squeeze_(0)
-    return x, local_l_mean, local_l_var, batch_index, y
+    x = tensors[REGISTRY_KEYS.X_KEY].squeeze_(0)
+    batch_index = tensors[REGISTRY_KEYS.BATCH_KEY].squeeze_(0)
+    y = tensors[REGISTRY_KEYS.LABELS_KEY].squeeze_(0)
+    return x, batch_index, y
 
 
 class GIMVI(VAEMixin, BaseModelClass):
@@ -39,10 +41,10 @@ class GIMVI(VAEMixin, BaseModelClass):
     Parameters
     ----------
     adata_seq
-        AnnData object that has been registered via :func:`~scvi.data.setup_anndata`
+        AnnData object that has been registered via :meth:`~scvi.external.GIMVI.setup_anndata`
         and contains RNA-seq data.
     adata_spatial
-        AnnData object that has been registered via :func:`~scvi.data.setup_anndata`
+        AnnData object that has been registered via :meth:`~scvi.external.GIMVI.setup_anndata`
         and contains spatial data.
     n_hidden
         Number of nodes per hidden layer.
@@ -59,8 +61,8 @@ class GIMVI(VAEMixin, BaseModelClass):
     --------
     >>> adata_seq = anndata.read_h5ad(path_to_anndata_seq)
     >>> adata_spatial = anndata.read_h5ad(path_to_anndata_spatial)
-    >>> scvi.data.setup_anndata(adata_seq)
-    >>> scvi.data.setup_anndata(adata_spatial)
+    >>> scvi.external.GIMVI.setup_anndata(adata_seq)
+    >>> scvi.external.GIMVI.setup_anndata(adata_spatial)
     >>> vae = scvi.model.GIMVI(adata_seq, adata_spatial)
     >>> vae.train(n_epochs=400)
 
@@ -81,14 +83,25 @@ class GIMVI(VAEMixin, BaseModelClass):
         **model_kwargs,
     ):
         super(GIMVI, self).__init__()
+        if adata_seq is adata_spatial:
+            raise ValueError(
+                "`adata_seq` and `adata_spatial` cannot point to the same object. "
+                "If you would really like to do this, make a copy of the object and pass it in as `adata_spatial`."
+            )
         self.adatas = [adata_seq, adata_spatial]
-        self.scvi_setup_dicts_ = {
-            "seq": adata_seq.uns["_scvi"],
-            "spatial": adata_spatial.uns["_scvi"],
+        self.adata_managers = {
+            "seq": self._get_most_recent_anndata_manager(adata_seq, required=True),
+            "spatial": self._get_most_recent_anndata_manager(
+                adata_spatial, required=True
+            ),
         }
+        self.registries_ = []
+        for adm in self.adata_managers.values():
+            self._register_manager_for_instance(adm)
+            self.registries_.append(adm.registry)
 
-        seq_var_names = _get_var_names_from_setup_anndata(adata_seq)
-        spatial_var_names = _get_var_names_from_setup_anndata(adata_spatial)
+        seq_var_names = adata_seq.var_names
+        spatial_var_names = adata_spatial.var_names
 
         if not set(spatial_var_names) <= set(seq_var_names):
             raise ValueError("spatial genes needs to be subset of seq genes")
@@ -98,17 +111,30 @@ class GIMVI(VAEMixin, BaseModelClass):
         ]
         spatial_gene_loc = np.concatenate(spatial_gene_loc)
         gene_mappings = [slice(None), spatial_gene_loc]
-        sum_stats = [d.uns["_scvi"]["summary_stats"] for d in self.adatas]
+        sum_stats = [adm.summary_stats for adm in self.adata_managers.values()]
         n_inputs = [s["n_vars"] for s in sum_stats]
 
-        total_genes = adata_seq.uns["_scvi"]["summary_stats"]["n_vars"]
+        total_genes = n_inputs[0]
 
         # since we are combining datasets, we need to increment the batch_idx
         # of one of the datasets
-        adata_seq_n_batches = adata_seq.uns["_scvi"]["summary_stats"]["n_batch"]
-        adata_spatial.obs["_scvi_batch"] += adata_seq_n_batches
+        adata_seq_n_batches = sum_stats[0]["n_batch"]
+        adata_spatial.obs[
+            self.adata_managers["spatial"]
+            .data_registry[REGISTRY_KEYS.BATCH_KEY]
+            .attr_key
+        ] += adata_seq_n_batches
 
-        n_batches = sum([s["n_batch"] for s in sum_stats])
+        n_batches = sum(s["n_batch"] for s in sum_stats)
+
+        library_log_means = []
+        library_log_vars = []
+        for adata_manager in self.adata_managers.values():
+            adata_library_log_means, adata_library_log_vars = _init_library_size(
+                adata_manager, n_batches
+            )
+            library_log_means.append(adata_library_log_means)
+            library_log_vars.append(adata_library_log_vars)
 
         self.module = JVAE(
             n_inputs,
@@ -116,6 +142,8 @@ class GIMVI(VAEMixin, BaseModelClass):
             gene_mappings,
             generative_distributions,
             model_library_size,
+            library_log_means,
+            library_log_vars,
             n_batch=n_batches,
             n_latent=n_latent,
             **model_kwargs,
@@ -173,9 +201,9 @@ class GIMVI(VAEMixin, BaseModelClass):
         )
         self.train_indices_, self.test_indices_, self.validation_indices_ = [], [], []
         train_dls, test_dls, val_dls = [], [], []
-        for i, ad in enumerate(self.adatas):
+        for i, adm in enumerate(self.adata_managers.values()):
             ds = DataSplitter(
-                ad,
+                adm,
                 train_size=train_size,
                 validation_size=validation_size,
                 batch_size=batch_size,
@@ -253,10 +281,6 @@ class GIMVI(VAEMixin, BaseModelClass):
             for tensors in scdl:
                 (
                     sample_batch,
-                    local_l_mean,
-                    local_l_var,
-                    batch_index,
-                    label,
                     *_,
                 ) = _unpack_tensors(tensors)
                 latent.append(
@@ -308,8 +332,6 @@ class GIMVI(VAEMixin, BaseModelClass):
             for tensors in scdl:
                 (
                     sample_batch,
-                    local_l_mean,
-                    local_l_var,
                     batch_index,
                     label,
                     *_,
@@ -345,6 +367,7 @@ class GIMVI(VAEMixin, BaseModelClass):
     def save(
         self,
         dir_path: str,
+        prefix: Optional[str] = None,
         overwrite: bool = False,
         save_anndata: bool = False,
         **anndata_write_kwargs,
@@ -360,6 +383,8 @@ class GIMVI(VAEMixin, BaseModelClass):
         ----------
         dir_path
             Path to a directory.
+        prefix
+            Prefix to prepend to saved file names.
         overwrite
             Overwrite existing data or not. If `False` and directory
             already exists at `dir_path`, error will be raised.
@@ -368,11 +393,6 @@ class GIMVI(VAEMixin, BaseModelClass):
         anndata_write_kwargs
             Kwargs for anndata write function
         """
-        # get all the user attributes
-        user_attributes = self._get_user_attributes()
-        # only save the public attributes with _ at the very end
-        user_attributes = {a[0]: a[1] for a in user_attributes if a[0][-1] == "_"}
-        # save the model state dict and the trainer state dict only
         if not os.path.exists(dir_path) or overwrite:
             os.makedirs(dir_path, exist_ok=overwrite)
         else:
@@ -381,27 +401,42 @@ class GIMVI(VAEMixin, BaseModelClass):
                     dir_path
                 )
             )
+
+        file_name_prefix = prefix or ""
+
+        seq_adata = self.adatas[0]
+        spatial_adata = self.adatas[1]
         if save_anndata:
-            dataset_names = ["seq", "spatial"]
-            for i in range(len(self.adatas)):
-                save_path = os.path.join(
-                    dir_path, "adata_{}.h5ad".format(dataset_names[i])
-                )
-                self.adatas[i].write(save_path)
-                varnames_save_path = os.path.join(
-                    dir_path, "var_names_{}.csv".format(dataset_names[i])
-                )
+            seq_save_path = os.path.join(dir_path, f"{file_name_prefix}adata_seq.h5ad")
+            seq_adata.write(seq_save_path)
 
-                var_names = self.adatas[i].var_names.astype(str)
-                var_names = var_names.to_numpy()
-                np.savetxt(varnames_save_path, var_names, fmt="%s")
+            spatial_save_path = os.path.join(
+                dir_path, f"{file_name_prefix}adata_spatial.h5ad"
+            )
+            spatial_adata.write(spatial_save_path)
 
-        model_save_path = os.path.join(dir_path, "model_params.pt")
-        attr_save_path = os.path.join(dir_path, "attr.pkl")
+        # save the model state dict and the trainer state dict only
+        model_state_dict = self.module.state_dict()
 
-        torch.save(self.module.state_dict(), model_save_path)
-        with open(attr_save_path, "wb") as f:
-            pickle.dump(user_attributes, f)
+        seq_var_names = seq_adata.var_names.astype(str).to_numpy()
+        spatial_var_names = spatial_adata.var_names.astype(str).to_numpy()
+
+        # get all the user attributes
+        user_attributes = self._get_user_attributes()
+        # only save the public attributes with _ at the very end
+        user_attributes = {a[0]: a[1] for a in user_attributes if a[0][-1] == "_"}
+
+        model_save_path = os.path.join(dir_path, f"{file_name_prefix}model.pt")
+
+        torch.save(
+            dict(
+                model_state_dict=model_state_dict,
+                seq_var_names=seq_var_names,
+                spatial_var_names=spatial_var_names,
+                attr_dict=user_attributes,
+            ),
+            model_save_path,
+        )
 
     @classmethod
     def load(
@@ -410,25 +445,28 @@ class GIMVI(VAEMixin, BaseModelClass):
         adata_seq: Optional[AnnData] = None,
         adata_spatial: Optional[AnnData] = None,
         use_gpu: Optional[Union[str, int, bool]] = None,
+        prefix: Optional[str] = None,
     ):
         """
         Instantiate a model from the saved output.
 
         Parameters
         ----------
+        dir_path
+            Path to saved outputs.
         adata_seq
             AnnData organized in the same way as data used to train model.
-            It is not necessary to run :func:`~scvi.data.setup_anndata`,
+            It is not necessary to run :meth:`~scvi.external.GIMVI.setup_anndata`,
             as AnnData is validated against the saved `scvi` setup dictionary.
-            AnnData must be registered via :func:`~scvi.data.setup_anndata`.
+            AnnData must be registered via :meth:`~scvi.external.GIMVI.setup_anndata`.
         adata_spatial
             AnnData organized in the same way as data used to train model.
             If None, will check for and load anndata saved with the model.
-        dir_path
-            Path to saved outputs.
         use_gpu
             Load model on default GPU if available (if None or True),
             or index of GPU to use (if int), or name of GPU (if str), or use CPU (if False).
+        prefix
+            Prefix of saved file names.
 
         Returns
         -------
@@ -439,31 +477,25 @@ class GIMVI(VAEMixin, BaseModelClass):
         >>> vae = GIMVI.load(adata_seq, adata_spatial, save_path)
         >>> vae.get_latent_representation()
         """
-        model_path = os.path.join(dir_path, "model_params.pt")
-        setup_dict_path = os.path.join(dir_path, "attr.pkl")
-        seq_data_path = os.path.join(dir_path, "adata_seq.h5ad")
-        spatial_data_path = os.path.join(dir_path, "adata_spatial.h5ad")
-        seq_var_names_path = os.path.join(dir_path, "var_names_seq.csv")
-        spatial_var_names_path = os.path.join(dir_path, "var_names_spatial.csv")
+        _, device = parse_use_gpu_arg(use_gpu)
 
-        if adata_seq is None and os.path.exists(seq_data_path):
-            adata_seq = read(seq_data_path)
-        elif adata_seq is None and not os.path.exists(seq_data_path):
-            raise ValueError(
-                "Save path contains no saved anndata and no adata was passed."
-            )
-        if adata_spatial is None and os.path.exists(spatial_data_path):
-            adata_spatial = read(spatial_data_path)
-        elif adata_spatial is None and not os.path.exists(spatial_data_path):
-            raise ValueError(
-                "Save path contains no saved anndata and no adata was passed."
-            )
-        adatas = [adata_seq, adata_spatial]
-
-        seq_var_names = np.genfromtxt(seq_var_names_path, delimiter=",", dtype=str)
-        spatial_var_names = np.genfromtxt(
-            spatial_var_names_path, delimiter=",", dtype=str
+        (
+            attr_dict,
+            seq_var_names,
+            spatial_var_names,
+            model_state_dict,
+            loaded_adata_seq,
+            loaded_adata_spatial,
+        ) = _load_saved_gimvi_files(
+            dir_path,
+            adata_seq is None,
+            adata_spatial is None,
+            prefix=prefix,
+            map_location=device,
         )
+        adata_seq = loaded_adata_seq or adata_seq
+        adata_spatial = loaded_adata_spatial or adata_spatial
+        adatas = [adata_seq, adata_spatial]
         var_names = [seq_var_names, spatial_var_names]
 
         for i, adata in enumerate(adatas):
@@ -476,12 +508,32 @@ class GIMVI(VAEMixin, BaseModelClass):
                     "need to be the same and in the same order as the adata used to train the model."
                 )
 
-        with open(setup_dict_path, "rb") as handle:
-            attr_dict = pickle.load(handle)
+        if "scvi_setup_dicts_" in attr_dict:
+            scvi_setup_dicts = attr_dict.pop("scvi_setup_dicts_")
+            for adata, scvi_setup_dict in zip(adatas, scvi_setup_dicts):
+                cls.register_manager(
+                    manager_from_setup_dict(cls, adata, scvi_setup_dict)
+                )
+        else:
+            registries = attr_dict.pop("registries_")
+            for adata, registry in zip(adatas, registries):
+                if (
+                    _MODEL_NAME_KEY in registry
+                    and registry[_MODEL_NAME_KEY] != cls.__name__
+                ):
+                    raise ValueError(
+                        "It appears you are loading a model from a different class."
+                    )
 
-        scvi_setup_dicts = attr_dict.pop("scvi_setup_dicts_")
-        transfer_anndata_setup(scvi_setup_dicts["seq"], adata_seq)
-        transfer_anndata_setup(scvi_setup_dicts["spatial"], adata_spatial)
+                if _SETUP_ARGS_KEY not in registry:
+                    raise ValueError(
+                        "Saved model does not contain original setup inputs. "
+                        "Cannot load the original setup."
+                    )
+
+                cls.setup_anndata(
+                    adata, source_registry=registry, **registry[_SETUP_ARGS_KEY]
+                )
 
         # get the parameters for the class init signiture
         init_params = attr_dict.pop("init_params_")
@@ -506,12 +558,42 @@ class GIMVI(VAEMixin, BaseModelClass):
         for attr, val in attr_dict.items():
             setattr(model, attr, val)
 
-        _, device = parse_use_gpu_arg(use_gpu)
-        model.module.load_state_dict(torch.load(model_path, map_location=device))
+        model.module.load_state_dict(model_state_dict)
         model.module.eval()
         model.to_device(device)
         return model
 
+    @classmethod
+    @setup_anndata_dsp.dedent
+    def setup_anndata(
+        cls,
+        adata: AnnData,
+        batch_key: Optional[str] = None,
+        labels_key: Optional[str] = None,
+        layer: Optional[str] = None,
+        **kwargs,
+    ):
+        """
+        %(summary)s.
+
+        Parameters
+        ----------
+        %(param_batch_key)s
+        %(param_labels_key)s
+        %(param_layer)s
+        """
+        setup_method_args = cls._get_setup_method_args(**locals())
+        anndata_fields = [
+            LayerField(REGISTRY_KEYS.X_KEY, layer, is_count_data=True),
+            CategoricalObsField(REGISTRY_KEYS.BATCH_KEY, batch_key),
+            CategoricalObsField(REGISTRY_KEYS.LABELS_KEY, labels_key),
+        ]
+        adata_manager = AnnDataManager(
+            fields=anndata_fields, setup_method_args=setup_method_args
+        )
+        adata_manager.register_fields(adata, **kwargs)
+        cls.register_manager(adata_manager)
+
 
 class TrainDL(DataLoader):
     def __init__(self, data_loader_list, **kwargs):
