@@ -119,7 +119,8 @@ class BaseModelClass(metaclass=BaseModelMetaClass):
         self.module.to(my_device)
 
     @property
-    def device(self):
+    def device(self) -> str:
+        """The current device that the module's params are on."""
         return self.module.device
 
     @staticmethod
@@ -255,10 +256,12 @@ class BaseModelClass(metaclass=BaseModelMetaClass):
 
         adata_manager = cls._per_instance_manager_store[self.id][adata_id]
         if adata_manager.adata is not adata:
-            raise ValueError(
-                "The provided AnnData object does not match the AnnData object "
-                "previously provided for setup. Did you make a copy?"
+            logger.info(
+                "AnnData object appears to be a copy. Attempting to transfer setup."
             )
+            _assign_adata_uuid(adata, overwrite=True)
+            adata_manager = self.adata_manager.transfer_fields(adata)
+            self._register_manager_for_instance(adata_manager)
 
         return adata_manager
 
@@ -355,7 +358,7 @@ class BaseModelClass(metaclass=BaseModelMetaClass):
         if adata.is_view:
             if copy_if_view:
                 logger.info("Received view of anndata, making copy.")
-                adata = adata.copy()
+                adata._init_as_actual(adata.copy())
                 # Reassign AnnData UUID to produce a separate AnnDataManager.
                 _assign_adata_uuid(adata, overwrite=True)
             else:
@@ -391,19 +394,23 @@ class BaseModelClass(metaclass=BaseModelMetaClass):
                 raise RuntimeError(message)
 
     @property
-    def is_trained(self):
+    def is_trained(self) -> bool:
+        """Whether the model has been trained."""
         return self.is_trained_
 
     @property
-    def test_indices(self):
+    def test_indices(self) -> np.ndarray:
+        """Observations that are in test set."""
         return self.test_indices_
 
     @property
-    def train_indices(self):
+    def train_indices(self) -> np.ndarray:
+        """Observations that are in train set."""
         return self.train_indices_
 
     @property
-    def validation_indices(self):
+    def validation_indices(self) -> np.ndarray:
+        """Observations that are in validation set."""
         return self.validation_indices_
 
     @train_indices.setter
@@ -584,7 +591,16 @@ class BaseModelClass(metaclass=BaseModelMetaClass):
         # Legacy support for old setup dict format.
         if "scvi_setup_dict_" in attr_dict:
             scvi_setup_dict = attr_dict.pop("scvi_setup_dict_")
-            cls.register_manager(manager_from_setup_dict(cls, adata, scvi_setup_dict))
+            unlabeled_category_key = "unlabeled_category_"
+            unlabeled_category = attr_dict.get(unlabeled_category_key, None)
+            cls.register_manager(
+                manager_from_setup_dict(
+                    cls,
+                    adata,
+                    scvi_setup_dict,
+                    unlabeled_category=unlabeled_category,
+                )
+            )
         else:
             registry = attr_dict.pop("registry_")
             if (
