@@ -11,6 +11,7 @@ from anndata import AnnData
 
 from scvi import REGISTRY_KEYS
 from scvi._compat import Literal
+from scvi._types import Number
 from scvi._utils import _doc_params
 from scvi.utils._docstrings import doc_differential_expression
 
@@ -19,8 +20,6 @@ from ._utils import _de_core
 
 logger = logging.getLogger(__name__)
 
-Number = Union[int, float]
-
 
 class RNASeqMixin:
     """General purpose methods for RNA-seq analysis."""
